@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2015, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2017, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -101,7 +101,7 @@ void search_output_results(int hit_count,
   pthread_mutex_lock(&mutex_output);
 
   /* show results */
-  long toreport = MIN(opt_maxhits, hit_count);
+  int64_t toreport = MIN(opt_maxhits, hit_count);
 
   if (fp_alnout)
     results_show_alnout(fp_alnout,
@@ -172,7 +172,7 @@ void search_output_results(int hit_count,
                                        qsequence_rc);
         }
     }
-  else if (opt_output_no_hits)
+  else
     {
       if (fp_uc)
         results_show_uc_one(fp_uc,
@@ -183,21 +183,24 @@ void search_output_results(int hit_count,
                             qsequence_rc,
                             0);
       
-      if (fp_userout)
-        results_show_userout_one(fp_userout,
-                                 0,
-                                 query_head, 
-                                 qsequence,
-                                 qseqlen,
-                                 qsequence_rc);
-      
-      if (fp_blast6out)
-        results_show_blast6out_one(fp_blast6out,
-                                   0,
-                                   query_head,
-                                   qsequence,
-                                   qseqlen,
-                                   qsequence_rc);
+      if (opt_output_no_hits)
+        {
+          if (fp_userout)
+            results_show_userout_one(fp_userout,
+                                     0,
+                                     query_head,
+                                     qsequence,
+                                     qseqlen,
+                                     qsequence_rc);
+
+          if (fp_blast6out)
+            results_show_blast6out_one(fp_blast6out,
+                                       0,
+                                       query_head,
+                                       qsequence,
+                                       qseqlen,
+                                       qsequence_rc);
+        }
     }
 
   if (hit_count)
@@ -225,7 +228,7 @@ void search_output_results(int hit_count,
   pthread_mutex_unlock(&mutex_output);
 }
 
-int search_query(long t)
+int search_query(int64_t t)
 {
   for (int s = 0; s < opt_strand; s++)
     {
@@ -264,14 +267,14 @@ int search_query(long t)
   /* free memory for alignment strings */
   for(int i=0; i<hit_count; i++)
     if (hits[i].aligned)
-      free(hits[i].nwalignment);
+      xfree(hits[i].nwalignment);
 
-  free(hits);
+  xfree(hits);
 
   return hit_count;
 }
 
-void search_thread_run(long t)
+void search_thread_run(int64_t t)
 {
   while (1)
     {
@@ -320,7 +323,7 @@ void search_thread_run(long t)
           strcpy(si_plus[t].qsequence, qseq);
           
           /* get progress as amount of input file read */
-          unsigned long progress = fasta_get_position(query_fasta_h);
+          uint64_t progress = fasta_get_position(query_fasta_h);
 
           /* let other threads read input */
           pthread_mutex_unlock(&mutex_input);
@@ -400,20 +403,20 @@ void search_thread_exit(struct searchinfo_s * si)
   nw_exit(si->nw);
 #endif
   unique_exit(si->uh);
-  free(si->hits);
+  xfree(si->hits);
   minheap_exit(si->m);
-  free(si->kmers);
+  xfree(si->kmers);
   if (si->query_head)
-    free(si->query_head);
+    xfree(si->query_head);
   if (si->qsequence)
-    free(si->qsequence);
+    xfree(si->qsequence);
 }
 
 
 
 void * search_thread_worker(void * vp)
 {
-  long t = (long) vp;
+  int64_t t = (int64_t) vp;
   search_thread_run(t);
   return 0;
 }
@@ -432,7 +435,7 @@ void search_thread_worker_run()
       if (si_minus)
         search_thread_init(si_minus+t);
       if (pthread_create(pthread+t, &attr,
-                         search_thread_worker, (void*)(long)t))
+                         search_thread_worker, (void*)(int64_t)t))
         fatal("Cannot create thread");
     }
 
@@ -641,10 +644,10 @@ void usearch_global(char * cmdline, char * progheader)
   pthread_mutex_destroy(&mutex_output);
   pthread_mutex_destroy(&mutex_input);
 
-  free(pthread);
-  free(si_plus);
+  xfree(pthread);
+  xfree(si_plus);
   if (si_minus)
-    free(si_minus);
+    xfree(si_minus);
 
   fasta_close(query_fasta_h);
 
@@ -678,7 +681,7 @@ void usearch_global(char * cmdline, char * progheader)
 
   if (opt_dbmatched || opt_dbnotmatched)
     {
-      for(long i=0; i<seqcount; i++)
+      for(int64_t i=0; i<seqcount; i++)
         if (dbmatched[i])
           {
             if (opt_dbmatched)
@@ -696,7 +699,7 @@ void usearch_global(char * cmdline, char * progheader)
           }
     }
 
-  free(dbmatched);
+  xfree(dbmatched);
 
   if (opt_dbmatched)
     fclose(fp_dbmatched);
