@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2015, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2017, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -90,7 +90,7 @@ void shuffle()
       deck[j] = t;
 
       passed++;
-      progress_update(i);
+      progress_update(passed);
     }
   progress_done();
   show_rusage();
@@ -107,7 +107,7 @@ void shuffle()
 
   show_rusage();
 
-  free(deck);
+  xfree(deck);
   db_free();
   fclose(fp_output);
 }
